// Sprite.cpp: implementation of the CSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DirectDraw.h"
#include "Sprite.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CSprite::CSprite()
{
	m_pDisplay = GetControlObject()->GetDisplay();
	m_dwState  = SPRITESTATE_STANDARD;
	m_ptPos    = CPoint(0, 0);
	m_iVelX    = 0;
	m_iVelY    = 0;
	m_iMaxVel  = DEFAULT_MAXVEL;
	m_iWidth   = 0;
	m_iHeight  = 0;
	//
	// Create static sprite
	m_iPhasesCount		= 1;	// One phase
	m_iAnimationSpeed	= 0;
	m_iCurrentPhase		= 0;
}

CSprite::~CSprite()
{

}

HRESULT		CSprite::CreateStaticSprite(LPSTR szSpriteBitmap, CPoint ptInitPos)
{
	DWORD	dwRet = ERROR_SUCCESS;
	DDSURFACEDESC2 dsdesc;
	dsdesc.dwSize = sizeof(DDSURFACEDESC2);
	//
	// Set position
	m_ptPos = ptInitPos;
	m_csBitmap = szSpriteBitmap;
	//
	// Create surface for sprite
	dwRet = m_pDisplay->CreateSurfaceFromBitmap(&m_psurSpriteSurface, szSpriteBitmap, 0, 0);
	if(dwRet != DD_OK) {
		TRACE("Cannot create sprite surface due %d\n", dwRet);
		return dwRet;
	}
	//
	// Set color key on surface. Black transparent color.
	dwRet = m_psurSpriteSurface->SetColorKey(0);
	if(dwRet != DD_OK) {
		TRACE("Cannot set color key on surface due %d\n", dwRet);
		return dwRet;
	}
	//
	// Get width and height of created surface
	dwRet = m_psurSpriteSurface->GetDDrawSurface()->GetSurfaceDesc(&dsdesc);
	if(dwRet != DD_OK) {
		TRACE("Cannot get surface info due %d\n", dwRet);
		return dwRet;
	}
	//
	// Set width and height members of sprite
	m_iWidth = dsdesc.dwWidth / m_iPhasesCount;
	m_iHeight = dsdesc.dwHeight;

	return dwRet;
}

HRESULT		CSprite::CreateAnimatedSprite(LPSTR szSpriteBitmap, CPoint ptInitPos, int iPhasesCount, int	iAnimationSpeed)
{
	DWORD	dwRet = ERROR_SUCCESS;
	//
	// Set animaiont atributes
	m_iPhasesCount = iPhasesCount;
	m_iAnimationSpeed = iAnimationSpeed;
	//
	// Create static sprite
	dwRet = CreateStaticSprite(szSpriteBitmap, ptInitPos);
	if(dwRet != DD_OK) {
		TRACE("Cannot create static part of sprite due %d\n", dwRet);	
	}
	return dwRet;
}

HRESULT		CSprite::DrawSprite()
{
	DWORD	dwRet = ERROR_SUCCESS;
	CRect	rcSrc;
	//
	// Set source rect
	rcSrc.left = m_iCurrentPhase * m_iWidth;
	rcSrc.right = rcSrc.left + m_iWidth;	
	rcSrc.top = 0;
	rcSrc.bottom = m_iHeight;	
	//
	// Draw sprite at specified location
	dwRet = m_pDisplay->ColorKeyBlt(m_ptPos.x, m_ptPos.y, m_psurSpriteSurface->GetDDrawSurface(), &rcSrc);	
	if(dwRet != DD_OK) {		
		if(dwRet == DDERR_SURFACELOST) {			
			m_psurSpriteSurface->GetDDrawSurface()->Restore();
			m_psurSpriteSurface->DrawBitmap((TCHAR*)(LPCSTR)m_csBitmap, 0, 0);
		}
		else {
			TRACE("Cannot render sprite due %d\n", dwRet);
		}
	}
	return dwRet;
}

HRESULT		CSprite::MoveSprite()
{
	DWORD	dwRet = ERROR_SUCCESS;

	m_ptPos += CPoint(m_iVelX, m_iVelY);

	return dwRet;
}

HRESULT		CSprite::ChangeAnimationPhase()
{
	DWORD	dwRet = ERROR_SUCCESS;
	DWORD   dwNewTime = GetTickCount();
	//
	// Wait for right time to change phase
	if(int(dwNewTime - m_dwLastUpdate) > m_iAnimationSpeed) {
		//
		// Increment phase
		m_iCurrentPhase++;
		//
		// Check upper bound
		if(m_iCurrentPhase == m_iPhasesCount) {
			m_iCurrentPhase = 0;
		}
		// Remember last time of update
		m_dwLastUpdate = dwNewTime;
	}
	return dwRet;
}
