// Input1.cpp: implementation of the CInput class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Input1.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInput::CInput()
{
	m_bInit = FALSE;

	m_lpDI = NULL;
	m_lpDIDKeyboard = NULL;
	m_lpDIDMouse = NULL;
}

CInput::~CInput()
{
	Clean();
}

void CInput::Clean()
{
	if(m_bInit) {

		if(m_lpDIDKeyboard) 
			m_lpDIDKeyboard->Release();

		if(m_lpDIDMouse) 
			m_lpDIDMouse->Release();

		if(m_lpDI) 
			m_lpDI->Release();

		m_bInit = FALSE;
	}
}

HRESULT CInput::CreateDirectInputSystem(HINSTANCE hInst, HWND hWnd, CSize csResolution)
{
	HRESULT dwRet = 1;
	//
	// You cannot call this function twice
	if(!m_bInit) {
		//
		// Creation of direct input object
		dwRet = DirectInput8Create(hInst, DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID**)&m_lpDI, NULL);		
		if(dwRet != DI_OK) {
			TRACE("Cannot create direct input object %d\n", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Create keyboard object
		dwRet = m_lpDI->CreateDevice(GUID_SysKeyboard, &m_lpDIDKeyboard, NULL);		
		if(dwRet != DI_OK) {
			TRACE("Cannot create keyboard object %d\n", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Create mouse object
		dwRet = m_lpDI->CreateDevice(GUID_SysMouse, &m_lpDIDMouse, NULL);		
		if(dwRet != DI_OK) {
			TRACE("Cannot create mouse object %d\n", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Set data format for both devices
		dwRet = m_lpDIDMouse->SetDataFormat(&c_dfDIMouse);		
		if(dwRet != DI_OK) {
			TRACE("Cannot set data format for mouse %d\n", dwRet);
			Clean();
			return dwRet;
		}
		dwRet = m_lpDIDKeyboard->SetDataFormat(&c_dfDIKeyboard);		
		if(dwRet != DI_OK) {
			TRACE("Cannot set data format for keyboard %d\n", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Set cooperative level for both devices
		dwRet = m_lpDIDKeyboard->SetCooperativeLevel(hWnd, DISCL_FOREGROUND|DISCL_NONEXCLUSIVE);		
		if(dwRet != DI_OK) {
			TRACE("Cannot set cooperative level for keyboard %d\n", dwRet);
			Clean();
			return dwRet;
		}	
		dwRet = m_lpDIDMouse->SetCooperativeLevel(hWnd, DISCL_NONEXCLUSIVE|DISCL_FOREGROUND);		
		if(dwRet != DI_OK) {
			TRACE("Cannot set cooperative level for mouse %d\n", dwRet);
			Clean();
			return dwRet;
		}
		//	
		// Acquire mouse and keyboard
		dwRet = m_lpDIDKeyboard->Acquire();		
		if(dwRet != DI_OK) {
			TRACE("Cannot acquire keyboard %d\n", dwRet);
			Clean();
			return dwRet;
		}
		
		dwRet = m_lpDIDMouse->Acquire();		
		if(dwRet != DI_OK) {
			TRACE("Cannot acquire mouse %d\n", dwRet);
			Clean();
			return dwRet;
		}		
		//
		m_csResolution = csResolution;
		m_bInit = TRUE;
	}
	return dwRet;
}

